using Articy.Unity;
using Articy.Unity.Interfaces;
using System.Collections.Generic;
using UnityEngine;

public class ArticyStoryline : MonoBehaviour, IArticyFlowPlayerCallbacks {
    private IList<Branch> currentBranches;

    //previously ApplyTransition
    //is called at every "Pause" of the Flow Player (Pause = Dialogue Nodes,...)
    public void OnFlowPlayerPaused(IFlowObject currentNode) {
        var dialogueBox = gameObject.GetComponent<WorldSpaceDialogueBox>();

        if(currentNode != null) {
            var objWithText = currentNode as IObjectWithText;
            if(objWithText != null) {
                dialogueBox.SetText(objWithText.Text);
            }
            
            var objSpeaker = currentNode as IObjectWithSpeaker;
            var actor = string.Empty;
            if(objSpeaker != null) {
                var speakerDisplayName = objSpeaker.Speaker as IObjectWithDisplayName;
                actor = speakerDisplayName?.DisplayName ?? string.Empty;
            }
            dialogueBox.SetActor(actor);
        }
        else {
            dialogueBox.ClearUI();
        }
    }

    //is called with every new Dialogue Node and returns its branches
    public void OnBranchesUpdated(IList<Branch> branches) {
        currentBranches = branches;
        if(currentBranches.Count > 0) {
            gameObject.GetComponent<WorldSpaceDialogueBox>()
                        .SetTransitions(currentBranches);
        }
        else {
            Destroy(gameObject);
        }
    }

    //used by the Buttons in the DialogueBox script
    public void ApplyTransition(int count) {
        gameObject.GetComponent<WorldSpaceDialogueBox>().ClearUI();
        gameObject.GetComponent<ArticyFlowPlayer>()
                  .Play(currentBranches[count]);
    }
}